# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc


class DriverBase(object, metaclass=abc.ABCMeta):
    """Driver Base class.

    This class is base of monitoring hosts.
    """

    def __init__(self):
        pass

    @abc.abstractmethod
    def monitor_hosts(self):
        """Must override monitor_hosts method."""
        pass
